/* $Id: d_mos.model,v 25.95 2006/08/26 01:23:57 al Exp $ -*- C++ -*-
 * Copyright (C) 2001 Albert Davis
 * Author: Albert Davis <aldavis@ieee.org>
 *
 * This file is part of "Gnucap", the Gnu Circuit Analysis Package
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 *------------------------------------------------------------------
 * data structures and defaults for mos model.
 * internal units all mks (meters)
 * but some user input parameters are in cm.
 *
 * netlist syntax:
 * device:  mxxxx d g s b mname <device args> <model card args>
 * model:   .model mname NMOS <args>
 *	or  .model mname PMOS <args>
 */
/* This file is automatically generated. DO NOT EDIT */
#ifndef D_MOS_H_INCLUDED
#define D_MOS_H_INCLUDED

#include "d_diode.h"
  class DEV_MOS;		/* this file */
  class COMMON_MOS;
  class EVAL_MOS_Cgb;
  class EVAL_MOS_Cgd;
  class EVAL_MOS_Cgs;
  class DEV_DIODE;		/* external */
  class DEV_CAPACITANCE;
  class DEV_FPOLY_CAP;
  class DEV_CPOLY_G;
  class DEV_RESISTANCE;
  class MODEL_MOS_BASE;
  class SDP_MOS_BASE;
  enum {mNUM_INIT_COND = 3};
#include "u_sdp.h"
#include "e_node.h"
#include "e_subckt.h"
#include "e_model.h"
/*--------------------------------------------------------------------------*/
class COMMON_MOS
  :public COMMON_COMPONENT{
public:
  explicit COMMON_MOS(const COMMON_MOS& p);
  explicit COMMON_MOS(int c=0);
           ~COMMON_MOS();
  bool        operator==(const COMMON_COMPONENT&)const;
  COMMON_COMPONENT* clone()const {return new COMMON_MOS(*this);}
  void        parse(CS&);
  void        print(OMSTREAM&)const;
  void        elabo3(const COMPONENT*);
  const char* name()const {return "mosfet";}
  const SDP_CARD* sdp()const {return _sdp;}
  bool      has_sdp()const {return _sdp;}
  static int  count() {return _count;}
private: // strictly internal
  static int _count;
public: // input parameters
  PARAMETER<double> m;	// device multiplier
  PARAMETER<double> l_in;	// drawn (optical) channel length
  PARAMETER<double> w_in;	// channel width (drawn)
  PARAMETER<double> ad_in;	// drain area, drawn
  PARAMETER<double> as_in;	// source area, drawn
  PARAMETER<double> pd;	// drain perimeter
  PARAMETER<double> ps;	// source perimeter
  PARAMETER<double> nrd;	// drain # squares
  PARAMETER<double> nrs;	// source # squares
public: // calculated parameters
  const SDP_CARD* _sdp;
public: // attached commons
  COMMON_COMPONENT* _db;
  COMMON_COMPONENT* _sb;
};
/*--------------------------------------------------------------------------*/
class EVAL_MOS_Cgb : public COMMON_COMPONENT {
private:
  explicit EVAL_MOS_Cgb(const EVAL_MOS_Cgb& p)
    :COMMON_COMPONENT(p) {}
public:
  explicit EVAL_MOS_Cgb(int c=0) :COMMON_COMPONENT(c) {}
  bool operator==(const COMMON_COMPONENT& x)const {return COMMON_COMPONENT::operator==(x);}
  COMMON_COMPONENT* clone()const {return new EVAL_MOS_Cgb(*this);}
  const char* name()const {untested(); return "EVAL_MOS_Cgb";}
  void tr_eval(ELEMENT*d)const;
  bool has_tr_eval()const {return true;}
  bool has_ac_eval()const {return false;}
};
/*--------------------------------------------------------------------------*/
class EVAL_MOS_Cgd : public COMMON_COMPONENT {
private:
  explicit EVAL_MOS_Cgd(const EVAL_MOS_Cgd& p)
    :COMMON_COMPONENT(p) {}
public:
  explicit EVAL_MOS_Cgd(int c=0) :COMMON_COMPONENT(c) {}
  bool operator==(const COMMON_COMPONENT& x)const {return COMMON_COMPONENT::operator==(x);}
  COMMON_COMPONENT* clone()const {return new EVAL_MOS_Cgd(*this);}
  const char* name()const {untested(); return "EVAL_MOS_Cgd";}
  void tr_eval(ELEMENT*d)const;
  bool has_tr_eval()const {return true;}
  bool has_ac_eval()const {return false;}
};
/*--------------------------------------------------------------------------*/
class EVAL_MOS_Cgs : public COMMON_COMPONENT {
private:
  explicit EVAL_MOS_Cgs(const EVAL_MOS_Cgs& p)
    :COMMON_COMPONENT(p) {}
public:
  explicit EVAL_MOS_Cgs(int c=0) :COMMON_COMPONENT(c) {}
  bool operator==(const COMMON_COMPONENT& x)const {return COMMON_COMPONENT::operator==(x);}
  COMMON_COMPONENT* clone()const {return new EVAL_MOS_Cgs(*this);}
  const char* name()const {untested(); return "EVAL_MOS_Cgs";}
  void tr_eval(ELEMENT*d)const;
  bool has_tr_eval()const {return true;}
  bool has_ac_eval()const {return false;}
};
/*--------------------------------------------------------------------------*/
class DEV_MOS : public BASE_SUBCKT {
private:
  explicit DEV_MOS(const DEV_MOS& p);
public:
  explicit DEV_MOS();
           ~DEV_MOS() {--_count;}
private: // override virtual
  char      id_letter()const     {return 'M';}
  const char* dev_type()const    {return "mosfet";}
  int       max_nodes()const     {return 4;}
  int       min_nodes()const     {return 4;}
  int       out_nodes()const     {return 4;}
  //int     matrix_nodes()const; //BASE_SUBCKT
  int       net_nodes()const     {return 4;}
  int       int_nodes()const     {return 2;}
  CARD*     clone()const         {return new DEV_MOS(*this);}
  //void    parse_spice(CS&);    //BASE_SUBCKT
  //void    print_spice(OMSTREAM&,int)const; //BASE_SUBCKT
  void      elabo1();
  //void    map_nodes();         //BASE_SUBCKT
  //void    precalc();           //BASE_SUBCKT
  //void    dc_begin();          //BASE_SUBCKT
  //void    tr_begin();          //BASE_SUBCKT
  //void    tr_restore();        //BASE_SUBCKT
  void      dc_advance() {set_not_converged(); BASE_SUBCKT::dc_advance();}
  void      tr_advance() {set_not_converged(); BASE_SUBCKT::tr_advance();}
  bool      tr_needs_eval()const;
  void      tr_queue_eval()      {if(tr_needs_eval()){q_eval();}}
  bool      do_tr();
  //void    tr_load();           //BASE_SUBCKT
  //double  tr_review();         //BASE_SUBCKT
  //void    tr_accept();         //BASE_SUBCKT
  //void    tr_unload();         //BASE_SUBCKT
  double    tr_probe_num(CS&)const;
  //void    ac_begin();          //BASE_SUBCKT
  //void    do_ac();             //BASE_SUBCKT
  //void    ac_load();           //BASE_SUBCKT
  //XPROBE  ac_probe_ext(CS&)const;//CKT_BASE/nothing
public:
  static int  count() {return _count;}
public: // may be used by models
  void reverse_if_needed();
private: // not available even to models
  static int _count;
public: // input parameters
public: // calculated parameters
  double ids;	// 
  double idsxxx;	// 
  double gds;	// dids/dvds
  double gmf;	// dids/dvgs
  double gmr;	// dids/dvgd
  double gmbf;	// dids/dvbs
  double gmbr;	// dids/dvbd
  double idb;	// 
  double idbxxx;	// 
  double gdbdb;	// placeholder
  double gdbds;	// disub/dvds
  double gdbgs;	// disub/dvgs
  double gdbbs;	// disub/dvbs
  double isb;	// 
  double isbxxx;	// 
  double gsbsb;	// placeholder
  double gsbsd;	// disub/dvds
  double gsbgd;	// disub/dvgs
  double gsbbd;	// disub/dvbs
  double qgate;	// raw
  double cgs;	// dqgate_vgs placeholder
  double cggb;	// dqgate_vgb
  double cgsb;	// dqgate_vsb
  double cgdb;	// dqgate_vdb
  double qgs;	// forward mode
  double cgsgs;	// dqgs_vgs placeholder
  double cgsgb;	// dqgs_vgb
  double cgssb;	// dqgs_vsb
  double cgsdb;	// dqgs_vdb
  double qgd;	// reverse mode
  double cgdgd;	// dqgd_vgs placeholder
  double cgdgb;	// dqgd_vgb
  double cgdsb;	// dqgd_vsb
  double cgddb;	// dqgd_vdb
  double qdrn;	// Qds
  double cdsds;	// dqds_vds placeholder
  double cdgb;	// dqds_vgb
  double cdsb;	// dqds_vsb
  double cddb;	// dqds_vdb
  double qbulk;	// raw
  double cbs;	// dqbs_vbs placeholder
  double cbgb;	// dqbs_vbg
  double cbsb;	// dqbs_vsb
  double cbdb;	// dqbs_vdb
  double qbs;	// Qbs forward
  double cbsbs;	// dqbs_vbs placeholder
  double cbsgb;	// dqbs_vbg
  double cbssb;	// dqbs_vsb
  double cbsdb;	// dqbs_vdb
  double qbd;	// Qbd reverse
  double cbdbd;	// dqbd_vbd placeholder
  double cbdgb;	// dqbd_vbg
  double cbdsb;	// dqbd_vsb
  double cbddb;	// dqbd_vdb
  double gtau;	// 
  double cqgb;	// 
  double cqsb;	// 
  double cqdb;	// 
  double cqbb;	// 
  double vgs;	// terminal voltages
  double vds;	// 
  double vbs;	// 
  double vdsat;	// saturation voltage
  double vgst;	// vgs - von.
  double von;	// actual threshold voltage
  bool reversed;	// flag: Vgs < 0, reverse s & d
  bool cutoff;	// flag: in cut off region
  bool subthreshold;	// flag: subthreshold region
  bool saturated;	// flag: in saturation region
  bool sbfwd;	// flag: sb diode fwd biased
  bool punchthru;	// flag: punch thru region
public: // netlist
  DEV_RESISTANCE* _Rs;
  DEV_RESISTANCE* _Rd;
  DEV_DIODE* _Ddb;
  DEV_DIODE* _Dsb;
  DEV_CAPACITANCE* _Cgs;
  DEV_CAPACITANCE* _Cgd;
  DEV_CAPACITANCE* _Cgb;
  DEV_CPOLY_G* _Ids;
  DEV_CPOLY_G* _Idb;
  DEV_CPOLY_G* _Isb;
private: // node list
  enum {n_drain=0, n_gate=1, n_source=2, n_bulk=3, n_idrain=-1, n_isource=-2};
  node_t _nodes[6];
};
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
#endif
